#define WIN32_LEAN_AND_MEAN
#include<windows.h>
#include<tchar.h>
#include<d3d9.h>

#pragma comment(linker, "/SUBSYSTEM:WINDOWS")
#pragma comment(linker, "/ENTRY:DllMain")
#pragma comment(linker, "/MERGE:.rdata=.text")

#define __DBGMSG(text) OutputDebugString(_TEXT(text))

CRITICAL_SECTION cs; // Just in case
HINSTANCE hpixo = 0;

typedef IDirect3D9* (__stdcall *pfnDirect3DCreate9)(UINT);

IDirect3D9* __stdcall Direct3DCreate9(UINT version)
{
	const TCHAR* libs[] = {TEXT("dx9pixo.dll"), TEXT("pixo9.dll"), NULL};
	const char* funcs[] = {"_PixoDirect3DCreate9@4", "PixoDirect3DCreate9"};
	size_t i = 0;

	EnterCriticalSection(&cs);

	__DBGMSG("Direct3DCreate9 called");

	while(TRUE)
	{
		if (!libs[i])
		{
			__DBGMSG("Failed to load pixomatic");
			LeaveCriticalSection(&cs);
			return 0;
		}

		hpixo = LoadLibrary(libs[i]);

		if (hpixo) break;

		++i;
	}

	pfnDirect3DCreate9 pfn = (pfnDirect3DCreate9)GetProcAddress(hpixo, funcs[i]);
   
	if (!pfn)
	{
		__DBGMSG("Failed import function");
		LeaveCriticalSection(&cs);
		return 0;
	}

	IDirect3D9* ptr = pfn(version);

	LeaveCriticalSection(&cs);

	if (!ptr)
		__DBGMSG("Failed to create d3d9");
	else
		__DBGMSG("D3d9 created");

	return ptr;
}

int __stdcall DebugSetMute()
{
	return 1;
}

BOOL APIENTRY DllMain(HANDLE hmodule, DWORD reason, LPVOID )
{
	switch (reason)
	{
	case DLL_PROCESS_ATTACH:
		DisableThreadLibraryCalls((HMODULE)hmodule);
		InitializeCriticalSection(&cs);
		break;

	case DLL_PROCESS_DETACH:
		DeleteCriticalSection(&cs);
		break;
	}

	return TRUE;
}